@@ -41,7 +41,7 @@ import gtk, gconf
 import json, uuid
 import gwibber.lib
 import gwibber.gwui
-import gwibber.util
+import gwibber.microblog.util
 from gwibber.lib.gtk import *
 
 from microblog.util.const import *
@@ -77,7 +77,7 @@ class GwibberAccountManager(object):
 
         self.protocols = json.loads(self.pub.GetServices())
 
-        self.gwibber_service = gwibber.util.getbus("Service")
+        self.gwibber_service = gwibber.microblog.util.getbus("Service")
 
         # FIXME: this should check for configured accounts, and if there are any hide this
         self.ui.get_object('frame_new_account').hide()
@@ -124,7 +124,7 @@ class GwibberAccountManager(object):
             self.add_account()
             return
 	for self.account in self.accounts:
-            if self.account_verify(self.account):
+            #if self.account_verify(self.account):
                 try:
                     icf = resources.icon(self.account["protocol"], use_theme=False) or resources.get_ui_asset("icons/breakdance/16x16/%s.png" % self.account["protocol"])
                     icon = gtk.gdk.pixbuf_new_from_file(icf)
@@ -133,21 +133,20 @@ class GwibberAccountManager(object):
                     icon = gtk.icon_theme_add_builtin_icon("gwibber", 22,
                                                            gtk.gdk.pixbuf_new_from_file_at_size(
                                                            resources.get_ui_asset("gwibber.svg"), 16, 16))
-                has_password = True
+                has_secret = True
                 for config in self.protocols[self.account["protocol"]]["config"]:
                     is_private = config.startswith("private:")
                     config = config.replace("private:", "")
-                    value = self.account[config]
                     if is_private:
+                        if not self.account.has_key(config): self.account[config] = ":KEYRING:MISSING"
                         if self.account[config].startswith(":KEYRING:"):
                             try:
                                 value = gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, {"id": str("%s/%s" % (self.account["_id"], config))})[0].secret
                             except gnomekeyring.NoMatchError:
-                                has_password = False
-                                    
+                                has_secret = False
 
                 bg_color = None
-                if not has_password:
+                if not has_secret:
                     bg_color = "pink"
                 
                 accounts_store.append(None, [self.account["protocol"] + " (" + self.account["username"] + ")", icon, self.account, bg_color])
@@ -269,7 +268,7 @@ class GwibberAccountManager(object):
             image_type.set_from_pixbuf(icon)
         try:
           if not new_account:
-              self.aw = eval(protocol["name"]).AccountWidget(self.account)
+              self.aw = eval(protocol["name"]).AccountWidget(self.account, self.ui)
           else:
               self.aw = eval(protocol["name"]).AccountWidget(None, self.ui)
           self.aw.show()
