@@ -172,6 +172,6 @@ class tomboy(MessageAction):
 
   @classmethod
   def include(self, client, msg):
-    return util.service_is_running("org.gnome.Tomboy")
+    return gwibber.microblog.util.service_is_running("org.gnome.Tomboy")
 
 MENU_ITEMS = [reply, retweet, read, user, like, delete, tomboy]
