@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 
 import gtk, gobject, gwui, util, resources, actions, json, gconf
+import microblog.util
 import subprocess, os
 import time, datetime
 
@@ -32,11 +33,11 @@ class GwibberClient(gtk.Window):
     self.service.connect_to_signal("LoadingStarted", self.on_loading_started)
     self.service.connect_to_signal("LoadingComplete", self.on_loading_complete)
 
-    self.connection = util.getbus("Connection")
+    self.connection = microblog.util.getbus("Connection")
     self.connection.connect_to_signal("ConnectionOnline", self.on_connection_online)
     self.connection.connect_to_signal("ConnectionOffline", self.on_connection_offline)
 
-    util.getbus("Streams").connect_to_signal("SettingChanged", self.on_setting_changed)
+    microblog.util.getbus("Streams").connect_to_signal("SettingChanged", self.on_setting_changed)
 
     if len(json.loads(self.service.GetAccounts())) == 0:
       import upgrade
