@@ -1,6 +1,7 @@
 
 import os, json, urlparse, resources, util
 import gtk, gobject, pango, webkit, time, gconf
+import gwibber.microblog.util
 
 import gettext
 from gettext import lgettext as _
@@ -29,17 +30,17 @@ class Model(gobject.GObject):
   def __init__(self):
     gobject.GObject.__init__(self)
 
-    self.daemon = util.getbus("Service")
+    self.daemon = gwibber.microblog.util.getbus("Service")
 
-    self.account_monitor = util.getbus("Accounts")
+    self.account_monitor = gwibber.microblog.util.getbus("Accounts")
     self.account_monitor.connect_to_signal("AccountChanged", self.on_stream_changed)
     self.account_monitor.connect_to_signal("AccountDeleted", self.on_stream_changed)
 
-    self.streams_monitor = util.getbus("Streams")
+    self.streams_monitor = gwibber.microblog.util.getbus("Streams")
     self.streams_monitor.connect_to_signal("StreamChanged", self.on_stream_changed)
     self.streams_monitor.connect_to_signal("StreamClosed", self.on_stream_changed)
 
-    self.settings = util.SettingsMonitor()
+    self.settings = gwibber.microblog.util.SettingsMonitor()
     self.services = json.loads(self.daemon.GetServices())
     self.features = json.loads(self.daemon.GetFeatures())
     self.messages = CouchDatabase(COUCH_DB_MESSAGES, create=True)
@@ -162,7 +163,7 @@ class Model(gobject.GObject):
         for feature in default_streams:
           aname = self.features[feature]["stream"]
           item["items"].append({
-            "name": aname.capitalize(),
+            "name": _(aname.capitalize()),
             "account": aId,
             "stream": aname,
             "view": self.messages.execute_view("account_stream_time", "messages")[[aId, aname, {}]:[aId, aname, 0]],
@@ -264,7 +265,7 @@ class WebUi(webkit.WebView):
     default_font = self.gc.get_string("/desktop/gnome/interface/font_name")
     font_name, font_size = default_font.rsplit(None, 1)
     self.web_settings.set_property("sans-serif-font-family", font_name)
-    self.web_settings.set_property("default-font-size", int(font_size))
+    self.web_settings.set_property("default-font-size", float(font_size))
 
     if not resources.theme_exists(theme):
       theme = "default"
@@ -832,9 +833,9 @@ class InputTextView(gtk.TextView):
     self.overlay_color = util.get_theme_colors()["text"].darker(3).hex
     self.overlay_text = '<span weight="bold" size="xx-large" foreground="%s">%s</span>'
 
-    self.shortener = util.getbus("URLShorten")
+    self.shortener = gwibber.microblog.util.getbus("URLShorten")
 
-    self.connection = util.getbus("Connection")
+    self.connection = gwibber.microblog.util.getbus("Connection")
     self.connection.connect_to_signal("ConnectionOnline", self.on_connection_online)
     self.connection.connect_to_signal("ConnectionOffline", self.on_connection_offline)
 
