@@ -26,7 +26,7 @@ class AccountWidget(gtk.VBox):
   """AccountWidget: A widget that provides a user interface for configuring flickr accounts in Gwibber
   """
   
-  def __init__(self, account=None):
+  def __init__(self, account=None, dialog=None):
     """Creates the account pane for configuring flickr accounts"""
     gtk.VBox.__init__( self, False, 20 )
     self.ui = gtk.Builder()
@@ -36,4 +36,7 @@ class AccountWidget(gtk.VBox):
     self.vbox_settings = self.ui.get_object("vbox_settings")
     self.pack_start(self.vbox_settings, False, False)
     self.vbox_settings.show_all()
+    if dialog:
+      dialog.get_object("vbox_create").show()
+
     
