@@ -21,7 +21,7 @@
 from dbus.mainloop.glib import DBusGMainLoop
 import gobject, gtk
 from gwibber.microblog.util.const import *
-import gwibber.gwui, gwibber.util, gwibber.resources
+import gwibber.gwui, gwibber.microblog.util, gwibber.resources
 import gettext
 from gettext import lgettext as _
 if hasattr(gettext, 'bind_textdomain_codeset'):
@@ -33,7 +33,7 @@ class GwibberPosterVBox(gtk.VBox):
     gtk.VBox.__init__(self)
     DBusGMainLoop(set_as_default=True)
     loop = gobject.MainLoop()
-    self.service = gwibber.util.getbus("Service")
+    self.service = gwibber.microblog.util.getbus("Service")
     self.stream_model = gwibber.gwui.Model()
 
     self.input = gwibber.gwui.Input()
