@@ -81,15 +81,22 @@ def perform_operation((acctid, opname, args, transient)):
         if m.has_key("id"):
           key = (m["id"], m["account"], opname, transient)
           key = "-".join(x for x in key if x)
+          m["operation"] = opname
+          m["stream"] = stream
+          m["transient"] = transient
+          m["rtl"] = util.isRTL(re.sub(text_cleaner, "", m["text"].decode('utf-8')))
           if not messages.record_exists(key):
-            m["operation"] = opname
-            m["stream"] = stream
-            m["transient"] = transient
-            m["rtl"] = util.isRTL(re.sub(text_cleaner, "", m["text"].decode('utf-8')))
-        
             log.logger.debug("%s Adding record", logtext)
             new_messages.append(m)
             messages.put_record(CouchRecord(m, COUCH_TYPE_MESSAGE, key))
+          else:
+            if m.has_key("comments"):
+              try:
+                if m["comments"] > 0 and m["comments"] != dict(messages.get_record(key).items())["comments"]:
+                  log.logger.debug("%s Updating record", logtext)
+                  messages.update_fields(key, m)
+              except:
+                pass
 
     log.logger.debug("%s Finished operation", logtext)
     return ("Success", new_messages)
@@ -377,6 +384,8 @@ class Dispatcher(dbus.service.Object):
 
     self.RefreshCreds()
     
+    self.refresh_timer_id = None
+
     if autorefresh:
       self.refresh()
 
@@ -535,14 +544,16 @@ class Dispatcher(dbus.service.Object):
       log.logger.debug("*** Sending Message ***")
       MapAsync(perform_operation, operations, self.loading_complete, self.loading_failed)
 
-  def refresh(self):
-    log.logger.debug("Refresh interval is set to %s", SETTINGS["interval"])
+  def refresh(self): 
+    if self.refresh_timer_id:
+      gobject.source_remove(self.refresh_timer_id)
+    log.logger.info("Refresh interval is set to %s", SETTINGS["interval"])
     operations = list(self.collector.get_operations())
     if operations:
-      log.logger.debug("** Starting Refresh - %s **", time.asctime())
+      log.logger.info("** Starting Refresh - %s **", time.asctime())
       self.LoadingStarted()
       MapAsync(perform_operation, operations, self.loading_complete, self.loading_failed)
-    gobject.timeout_add(int(60 * 1000 * SETTINGS["interval"]), self.refresh)
+    self.refresh_timer_id = gobject.timeout_add_seconds(int(60 * SETTINGS["interval"]), self.refresh)
     return False
 
 class ConnectionMonitor(dbus.service.Object):
