@@ -1,9 +1,14 @@
 #!/usr/bin/env python
 
 import network, util
-from util import exceptions
+from util import exceptions, log
 import hashlib, mx.DateTime, time
+from os.path import join, getmtime, exists
 from gettext import lgettext as _
+from util.const import *
+from storage import MessageStore
+
+log.logger.name = "Facebook"
 
 PROTOCOL_INFO = {
   "name": "Facebook",
@@ -25,7 +30,6 @@ PROTOCOL_INFO = {
     "send",
     "reply",
     "receive",
-    "responses",
     "thread",
     "delete",
     "send_thread",
@@ -35,12 +39,10 @@ PROTOCOL_INFO = {
 
   "default_streams": [
     "receive",
-    "responses",
     "images",
   ]
 }
 
-APP_KEY = "71b85c6d8cb5bbb9f1a3f8bbdcdd4b05"
 URL_PREFIX = "https://api.facebook.com/restserver.php"
 POST_URL = "http://www.facebook.com/profile.php?id=%s&v=feed&story_fbid=%s&ref=mf"
 
@@ -48,13 +50,21 @@ class Client:
   def __init__(self, acct):
     self.account = acct
     self.user_id = acct["session_key"].split("-")[1]
+    self.msg_store = MessageStore()
 
+  def _check_error(self, data):
+    if isinstance(data, dict) and "error_code" in data:
+      log.logger.info("Facebook error %s - %s", data["error_code"], data["error_msg"])
+      return True
+    else: 
+      return False
+    
   def _get(self, operation, post=False, single=False, **args):
     args.update({
       "v": "1.0",
       "format": "json",
       "method": "facebook." + operation,
-      "api_key": APP_KEY,
+      "api_key": FB_APP_KEY,
       "session_key": self.account["session_key"],
       "call_id": str(int(time.time()) * 1000),
     })
@@ -62,11 +72,10 @@ class Client:
     sig = "".join("%s=%s" % (k, v) for k, v in sorted(args.items()))
     args["sig"] = hashlib.md5(sig + self.account["secret_key"]).hexdigest()
     data = network.Download(URL_PREFIX, args, post).get_json()
-    if "error_msg" in data:
-      if data.has_key("error_msg"):
-        if "permission" in data["error_msg"]:
-          raise exceptions.GwibberProtocolError("auth", self.account["protocol"], self.account["username"], data["error_msg"])
 
+    if isinstance(data, dict) and data.get("error_msg", 0):
+      if "permission" in data["error_msg"]:
+        raise exceptions.GwibberProtocolError("auth", self.account["protocol"], self.account["username"], data["error_msg"])
     return data
 
   def _sender(self, user):
@@ -78,7 +87,7 @@ class Client:
       "image": user["pic_square"],
     }
     
-    if not "?" in user["url"]:
+    if user["url"] and not "?" in user["url"]:
       sender["nick"] = user["url"].rsplit("/", 1)[-1]
     return sender
   
@@ -87,7 +96,7 @@ class Client:
     m["id"] = str(data["post_id"])
     m["protocol"] = "facebook"
     m["account"] = self.account["_id"]
-    m["time"] = int(mx.DateTime.DateTimeFrom(data['created_time']).gmtime())
+    m["time"] = int(mx.DateTime.DateTimeFrom(int(data.get("updated_time", data["created_time"]))).gmtime())
     m["url"] = data["permalink"]
 
     if data.get("attribution", 0):
@@ -114,11 +123,12 @@ class Client:
     if data.get("comments", 0):
       m["comments"] = []
       for item in data["comments"]["comment_list"]:
-        m["comments"].append({
-          "text": item["text"],
-          "time": int(mx.DateTime.DateTimeFrom(item["time"]).gmtime()),
-          "sender": self._sender(profiles[item["fromid"]]),
-        })
+        if item["fromid"] in profiles:
+          m["comments"].append({
+            "text": item["text"],
+            "time": int(mx.DateTime.DateTimeFrom(item["time"]).gmtime()),
+            "sender": self._sender(profiles[item["fromid"]]),
+          })
 
     if data.get("attachment", 0):
       if data["attachment"].get("name", 0):
@@ -162,7 +172,7 @@ class Client:
     }
 
   def _image(self, data, profiles):
-    user = profiles[data["owner"]]
+    user = profiles[int(data["owner"])]
     return {
       "id": str(data["object_id"]),
       "protocol": "facebook",
@@ -187,12 +197,33 @@ class Client:
     }
 
   def _friends(self):
-    friends = self._get("fql.query", query="""
-      SELECT name, profile_url, pic_square, username, uid
-        FROM user WHERE uid in (SELECT uid2 FROM friend WHERE uid1=%s)
-      """ % self.user_id)
+    friends_cache_file = join(CACHE_DIR, ("%s_friends.cache" % self.account["_id"]))
+    if not exists(friends_cache_file):
+      f = file(friends_cache_file, "w")
+      f.close()
+    f = open(friends_cache_file, "r")
+    try:
+      friends = eval(f.read())
+    except SyntaxError:
+      friends = ""
+    if (int(getmtime(friends_cache_file)) < int(mx.DateTime.DateTimeFromTicks(mx.DateTime.localtime()) - mx.DateTime.TimeDelta(hours=4.0))) \
+      or not isinstance(friends, list):
+      log.logger.debug("facebook:friends is refreshing at %s", mx.DateTime.localtime())
+      
+      f.close()
+      f = open(friends_cache_file, "r+")
+      friends = self._get("fql.query", query="""
+        SELECT name, profile_url, pic_square, username, uid
+          FROM user WHERE uid in (SELECT uid2 FROM friend WHERE uid1=%s)
+        """ % self.user_id)
+      f.write(str(friends))
+      f.close()
+      log.logger.debug("<STATS> facebook:friends account:%s size:%s", self.account["_id"], str(friends).__len__())
 
-    return dict((p["uid"], p) for p in friends)
+    if not self._check_error(friends):
+      return dict((p["uid"], p) for p in friends)
+    else:
+      return
 
   def __call__(self, opname, **args):
     return getattr(self, opname)(**args)
@@ -205,32 +236,53 @@ class Client:
     comments = self._get("stream.getComments", post_id=id)
     return [self._comment(comment, profiles) for comment in comments]
 
-  def receive(self):
-    data = self._get("stream.get", viewer_id=self.user_id, limit=80)
-    profiles = dict((p["id"], p) for p in data["profiles"])
-    return [self._message(post, profiles) for post in data["posts"]]
+  def receive(self, since=None):
+    if not since:
+      since = int(mx.DateTime.DateTimeFromTicks(mx.DateTime.localtime()) - mx.DateTime.TimeDelta(hours=240.0))
+    else:
+      since = int(mx.DateTime.DateTimeFromTicks(since).localtime())
+
+    data = self._get("stream.get", viewer_id=self.user_id, start_time=since, limit=80)
+    """
+    lastmsgtime = self.msg_store.get_last_message_time(self.account["_id"], "receive")
+    if not lastmsgtime:
+      lastmsgtime = int(mx.DateTime.DateTimeFromTicks(mx.DateTime.localtime()) - mx.DateTime.TimeDelta(hours=240.0))
+    lasttime = int(mx.DateTime.DateTimeFromTicks(lastmsgtime).localtime().ticks())
+    data = self._get("stream.get", viewer_id=self.user_id, start_time=lasttime, limit=80)
+    """
+    log.logger.debug("<STATS> facebook:receive account:%s since:%s size:%s", self.account["_id"], mx.DateTime.DateTimeFromTicks(since), str(data).__len__())
+    if not self._check_error(data):
+      profiles = dict((p["id"], p) for p in data["profiles"])
+      return [self._message(post, profiles) for post in data["posts"]]
+    else: return
+
+  def images(self, limit=100, since=None):
+    """
+    lastmsgtime = self.msg_store.get_last_message_time(self.account["_id"], "images")
+    if not lastmsgtime:
+      lastmsgtime = int(mx.DateTime.DateTimeFromTicks(mx.DateTime.localtime()) - mx.DateTime.TimeDelta(hours=240.0))
+    lasttime = int(mx.DateTime.DateTimeFromTicks(lastmsgtime).localtime().ticks())
+    """
+    if not since:
+      since = int(mx.DateTime.DateTimeFromTicks(mx.DateTime.localtime()) - mx.DateTime.TimeDelta(hours=240.0))
+    else:
+      since = int(mx.DateTime.DateTimeFromTicks(since).localtime())
 
-  def responses(self, limit=100):
-    data = self._get("fql.query", query="""
-      SELECT id, post_id, time, fromid, text, object_id FROM comment WHERE post_id IN
-        (SELECT post_id FROM stream WHERE source_id = %s) AND
-        fromid <> %s ORDER BY time DESC LIMIT 1,%s
-      """ % (self.user_id, self.user_id, limit))
-    
-    profiles = self._friends()
-    return [self._comment(comment, profiles) for comment in data]
 
-  def images(self, limit=100):
     data = self._get("fql.query", query="""
       SELECT owner, object_id, created, src_small, src_big, link, caption
         FROM photo WHERE aid in
         (SELECT aid FROM album WHERE owner IN
         (SELECT uid2 FROM friend WHERE uid1=%s))
-        ORDER BY created DESC LIMIT 1,%s
-      """ % (self.user_id, limit))
-    
-    profiles = self._friends()
-    return [self._image(post, profiles) for post in data]
+        AND created > %s
+        ORDER BY created DESC LIMIT %s
+      """ % (self.user_id, since, limit))
+
+    log.logger.debug("<STATS> facebook:images account:%s since:%s size:%s", self.account["_id"], mx.DateTime.DateTimeFromTicks(since), str(data).__len__())
+    if not self._check_error(data):
+      profiles = self._friends()
+      return [self._image(post, profiles) for post in data if int(post["owner"]) in profiles]
+    else: return
 
   def delete(self, message):
     self._get("stream.remove", post_id=message["id"])
