@@ -21,9 +21,19 @@ class CouchMessageStorage:
       reducefn = 'function(key, value, rereduce) { return Math.max.apply(Math, value); }'
       self.messages.add_view("maxid", viewfn, reducefn, "messages")
 
+    if not self.messages.view_exists("max_message_time", "messages"):
+      viewfn = 'function(doc) { emit([doc.account, doc.operation], doc.time); }'
+      reducefn = 'function(key, value, rereduce) { return Math.max.apply(Math, value); }'
+      self.messages.add_view("max_message_time", viewfn, reducefn, "messages")
+
   def get_highest_id(self, acctid, opname):
     output = self.messages.execute_view("maxid", "messages")[[acctid, opname]]
     results = output[[acctid, opname]].rows
     if len(results) > 0: return results[0].value
 
+  def get_last_message_time(self, acctid, opname):
+    output = self.messages.execute_view("max_message_time", "messages")[[acctid, opname]]
+    results = output[[acctid, opname]].rows
+    if len(results) > 0: return results[0].value
+
 MessageStore = CouchMessageStorage
