@@ -44,7 +44,6 @@ http://undefined.org/python/#simplejson to download it, or do
 apt-get install python-simplejson on a Debian-like system.
 """
 
-import md5
 import sys
 import time
 import urllib
@@ -52,6 +51,9 @@ import urllib2
 import httplib
 import mimetypes
 
+import pycurl
+import StringIO
+
 # try to use simplejson first, otherwise fallback to XML
 try:
     import simplejson
@@ -361,6 +363,59 @@ METHODS = {
             ('path', str, ['optional']),
         ],
     },
+
+    'stream' : {
+        'addComment' : [
+            ('post_id', int, []),
+            ('comment', str, []),
+            ('uid', int, ['optional']),
+        ],
+
+        'addLike': [
+            ('uid', int, ['optional']),
+            ('post_id', int, ['optional']),
+        ],
+            
+        'get' : [
+            ('viewer_id', int, ['optional']),
+            ('source_ids', list, ['optional']),
+            ('start_time', int, ['optional']),
+            ('end_time', int, ['optional']),
+            ('limit', int, ['optional']),
+            ('filter_key', str, ['optional']),
+        ],
+            
+        'getComments' : [
+            ('post_id', int, []),
+        ],
+            
+        'getFilters' : [
+            ('uid', int, ['optional']),
+        ],
+            
+        'publish' : [
+            ('message', str, ['optional']),
+            ('attachment', object, ['optional']),
+            ('action_links', list, ['optional']),
+            ('target_id', str, ['optional']),
+            ('uid', str, ['optional']),
+        ],    
+        
+        'remove' : [
+            ('post_id', int, []),
+            ('uid', int, ['optional']),
+        ],
+        
+        'removeComment' : [
+            ('comment_id', int, []),
+            ('uid', int, ['optional']),
+        ],
+        
+        'removeLike' : [
+            ('uid', int, ['optional']),
+            ('post_id', int, ['optional']),
+        ],
+    }
 }
 
 
@@ -506,7 +561,7 @@ class PhotosProxy(PhotosProxy):
             data = StringIO.StringIO()
             img.save(data, img.format)
 
-        content_type, body = self.__encode_multipart_formdata(list(args.iteritems()), [(image, data)])
+        content_type, body = self.__encode_multipart_formdata(list(args.items()), [(image, data)])
         h = httplib.HTTP('api.facebook.com')
         h.putrequest('POST', '/restserver.php')
         h.putheader('Content-Type', content_type)
@@ -655,7 +710,13 @@ class Facebook(object):
 
     def _hash_args(self, args, secret=None):
         """Hashes arguments by joining key=value pairs, appending a secret, and then taking the MD5 hex digest."""
-        hasher = md5.new(''.join(['%s=%s' % (x, args[x]) for x in sorted(args.keys())]))
+        try:
+            import hashlib
+            hasher = hashlib.md5(''.join(['%s=%s' % (x, args[x]) for x in sorted(args.keys())]))
+        except ImportError:
+            import md5
+            hasher = md5.new(''.join(['%s=%s' % (x, args[x]) for x in sorted(args.keys())]))
+
         if secret:
             hasher.update(secret)
         elif self.secret:
@@ -677,7 +738,7 @@ class Facebook(object):
             node.hasAttribute('list') and \
             node.getAttribute('list')=="true":
             return self._parse_response_list(node)
-        elif len(filter(lambda x: x.nodeType == x.ELEMENT_NODE, node.childNodes)) > 0:
+        elif len([x for x in node.childNodes if x.nodeType == x.ELEMENT_NODE]) > 0:
             return self._parse_response_dict(node)
         else:
             return ''.join(node.data for node in node.childNodes if node.nodeType == node.TEXT_NODE)
@@ -686,7 +747,7 @@ class Facebook(object):
     def _parse_response_dict(self, node):
         """Parses an XML dictionary response node from Facebook."""
         result = {}
-        for item in filter(lambda x: x.nodeType == x.ELEMENT_NODE, node.childNodes):
+        for item in [x for x in node.childNodes if x.nodeType == x.ELEMENT_NODE]:
             result[item.nodeName] = self._parse_response_item(item)
         if node.nodeType == node.ELEMENT_NODE and node.hasAttributes():
             if node.hasAttribute('id'):
@@ -697,14 +758,14 @@ class Facebook(object):
     def _parse_response_list(self, node):
         """Parses an XML list response node from Facebook."""
         result = []
-        for item in filter(lambda x: x.nodeType == x.ELEMENT_NODE, node.childNodes):
+        for item in [x for x in node.childNodes if x.nodeType == x.ELEMENT_NODE]:
             result.append(self._parse_response_item(item))
         return result
 
 
     def _check_error(self, response):
         """Checks if the given Facebook response is an error, and then raises the appropriate exception."""
-        if type(response) is dict and response.has_key('error_code'):
+        if isinstance(response, dict) and 'error_code' in response:
             raise FacebookError(response['error_code'], response['error_msg'], response['request_args'])
 
 
@@ -713,10 +774,10 @@ class Facebook(object):
         if args is None:
             args = {}
 
-        for arg in args.items():
-            if type(arg[1]) == list:
+        for arg in list(args.items()):
+            if isinstance(arg[1], list):
                 args[arg[0]] = ','.join(str(a) for a in arg[1])
-            elif type(arg[1]) == unicode:
+            elif isinstance(arg[1], unicode):
                 args[arg[0]] = arg[1].encode("UTF-8")
 
         args['method'] = method
@@ -768,12 +829,22 @@ class Facebook(object):
 
     def __call__(self, method, args=None, secure=True):
         """Make a call to Facebook's REST server."""
-        post_data = urllib.urlencode(self._build_post_args(method, args))
+        post_data = self._build_post_args(method, args)
 
-        if secure:
-            response = urllib2.urlopen(FACEBOOK_SECURE_URL, post_data).read()
-        else:
-            response = urllib2.urlopen(FACEBOOK_URL, post_data).read()
+        c = pycurl.Curl()
+        c.setopt(pycurl.URL, FACEBOOK_SECURE_URL if secure else FACEBOOK_URL)
+        c.setopt(pycurl.HTTPHEADER, ["Accept:"])
+        c.setopt(c.HTTPPOST, [(x,str(y)) for x,y in post_data.items()])
+
+        b = StringIO.StringIO()
+        c.setopt(pycurl.WRITEFUNCTION, b.write)
+        c.setopt(pycurl.FOLLOWLOCATION, 1)
+        c.setopt(pycurl.MAXREDIRS, 5)
+        c.setopt(pycurl.TIMEOUT, 15)
+        c.setopt(pycurl.CONNECTTIMEOUT, 8)
+        c.perform()
+
+        response = b.getvalue()
 
         return self._parse_response(response, method)
 
@@ -938,7 +1009,7 @@ class Facebook(object):
         if timeout and '%s_time' % prefix in post and time.time() - float(post['%s_time' % prefix]) > timeout:
             return None
 
-        args = dict([(key[len(prefix + '_'):], value) for key, value in args.items() if key.startswith(prefix)])
+        args = dict([(key[len(prefix + '_'):], value) for key, value in list(args.items()) if key.startswith(prefix)])
 
         hash = self._hash_args(args)
 
@@ -963,24 +1034,24 @@ if __name__ == '__main__':
     facebook.login()
 
     # Login to the window, then press enter
-    print 'After logging in, press enter...'
+    print('After logging in, press enter...')
     raw_input()
 
     facebook.auth.getSession()
-    print 'Session Key:   ', facebook.session_key
-    print 'Your UID:      ', facebook.uid
+    print('Session Key:   ', facebook.session_key)
+    print('Your UID:      ', facebook.uid)
 
     info = facebook.users.getInfo([facebook.uid], ['name', 'birthday', 'affiliations', 'sex'])[0]
 
-    print 'Your Name:     ', info['name']
-    print 'Your Birthday: ', info['birthday']
-    print 'Your Gender:   ', info['sex']
+    print('Your Name:     ', info['name'])
+    print('Your Birthday: ', info['birthday'])
+    print('Your Gender:   ', info['sex'])
 
     friends = facebook.friends.get()
     friends = facebook.users.getInfo(friends[0:5], ['name', 'birthday', 'relationship_status'])
 
     for friend in friends:
-        print friend['name'], 'has a birthday on', friend['birthday'], 'and is', friend['relationship_status']
+        print(friend['name'], 'has a birthday on', friend['birthday'], 'and is', friend['relationship_status'])
 
     arefriends = facebook.friends.areFriends([friends[0]['uid']], [friends[1]['uid']])
 
