@@ -1,6 +1,7 @@
-import network, util, htmllib
-from util import log
-from util import exceptions
+import network, util, htmllib, re
+import gnomekeyring
+from oauth import oauth
+from util import log, exceptions
 from gettext import lgettext as _
 log.logger.name = "Twitter"
 
@@ -9,14 +10,15 @@ PROTOCOL_INFO = {
   "version": "1.0",
   
   "config": [
-    "private:password",
+    "private:secret_token",
+    "access_token",
     "username",
     "color",
     "receive_enabled",
     "send_enabled",
   ],
  
-  "authtype": "login",
+  "authtype": "oauth1a",
   "color": "#729FCF",
 
   "features": [
@@ -44,6 +46,7 @@ PROTOCOL_INFO = {
 }
 
 URL_PREFIX = "https://twitter.com"
+API_PREFIX = "https://api.twitter.com/1"
 
 def unescape(s):
   p = htmllib.HTMLParser(None)
@@ -53,28 +56,40 @@ def unescape(s):
 
 class Client:
   def __init__(self, acct):
+    if not acct.has_key("access_token") and not acct.has_key("secret_token"):
+      raise exceptions.GwibberProtocolError("keyring")
+    if acct.has_key("secret_token") and acct.has_key("password"): acct.pop("password")
     self.account = acct
+    self.sigmethod = oauth.OAuthSignatureMethod_HMAC_SHA1()
+    self.consumer = oauth.OAuthConsumer(*util.resources.get_twitter_keys())
+    self.token = oauth.OAuthToken(acct["access_token"], acct["secret_token"])
 
   def _common(self, data):
     m = {}; 
-    m["id"] = str(data["id"])
-    m["protocol"] = "twitter"
-    m["account"] = self.account["_id"]
-    m["time"] = util.parsetime(data["created_at"])
-    m["text"] = unescape(data["text"])
-    m["to_me"] = ("@%s" % self.account["username"]) in data["text"]
+    try:
+      m["id"] = str(data["id"])
+      m["protocol"] = "twitter"
+      m["account"] = self.account["_id"]
+      m["time"] = util.parsetime(data["created_at"])
+      m["text"] = unescape(data["text"])
+      m["to_me"] = ("@%s" % self.account["username"]) in data["text"]
 
-    m["html"] = util.linkify(data["text"],
-      ((util.PARSE_HASH, '#<a class="hash" href="%s#search?q=\\1">\\1</a>' % URL_PREFIX),
-      (util.PARSE_NICK, '@<a class="nick" href="%s/\\1">\\1</a>' % URL_PREFIX)), escape=False)
-
-    m["content"] = util.linkify(data["text"],
-      ((util.PARSE_HASH, '#<a class="hash" href="gwibber:/tag?acct=%s&query=\\1">\\1</a>' % m["account"]),
-      (util.PARSE_NICK, '@<a class="nick" href="gwibber:/user?acct=%s&name=\\1">\\1</a>' % m["account"])), escape=False)
+      m["html"] = util.linkify(data["text"],
+        ((util.PARSE_HASH, '#<a class="hash" href="%s#search?q=\\1">\\1</a>' % URL_PREFIX),
+        (util.PARSE_NICK, '@<a class="nick" href="%s/\\1">\\1</a>' % URL_PREFIX)), escape=False)
 
+      m["content"] = util.linkify(data["text"],
+        ((util.PARSE_HASH, '#<a class="hash" href="gwibber:/tag?acct=%s&query=\\1">\\1</a>' % m["account"]),
+        (util.PARSE_NICK, '@<a class="nick" href="gwibber:/user?acct=%s&name=\\1">\\1</a>' % m["account"])), escape=False)
+    except: 
+      log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data)
+ 
     return m
 
   def _message(self, data):
+    if type(data) == type(None):
+      return []
+
     m = self._common(data)
     m["source"] = data.get("source", False)
     
@@ -122,15 +137,32 @@ class Client:
     return m
 
   def _get(self, path, parse="message", post=False, single=False, **args):
-    url = "/".join((URL_PREFIX, path))
+    url = "/".join((API_PREFIX, path))
+
+    request = oauth.OAuthRequest.from_consumer_and_token(self.consumer, self.token,
+        http_method=post and "POST" or "GET", http_url=url, parameters=util.compact(args))
+    request.sign_request(self.sigmethod, self.consumer, self.token)
     
-    data = network.Download(url, util.compact(args) or None, post,
-        self.account["username"], self.account["password"]).get_json()
-
-    if isinstance(data, dict) and data.get("error", 0):
-      if "authenticate" in data["error"]:
-        raise exceptions.GwibberProtocolError("auth", self.account["protocol"], self.account["username"], data["error"])
+    if post:
+      data = network.Download(request.to_url(), util.compact(args), post).get_json()
+    else:
+      data = network.Download(request.to_url(), None, post).get_json()
 
+    if isinstance(data, dict) and data.get("errors", 0):
+      if "authenticate" in data["errors"][0]["message"]:
+        raise exceptions.GwibberProtocolError("auth",
+            self.account["protocol"], self.account["username"], data["errors"][0]["message"])
+      else:
+        for error in data["errors"]:
+          log.logger.info("Twitter failure - %s", error["message"])
+        return []
+    elif isinstance(data, dict) and data.get("error", 0):
+      log.logger.error("%s failure - %s", PROTOCOL_INFO["name"], data["error"])
+      return []
+    elif isinstance(data, str):
+      log.logger.error("%s unexpected result - %s", PROTOCOL_INFO["name"], data)
+      return []
+    
     if single: return [getattr(self, "_%s" % parse)(data)]
     if parse: return [getattr(self, "_%s" % parse)(m) for m in data]
     else: return []
@@ -172,9 +204,8 @@ class Client:
 
   def send(self, message):
     return self._get("statuses/update.json", post=True, single=True,
-        status=message, source="gwibbernet")
-
+        status=message)
+  
   def send_thread(self, message, target):
     return self._get("statuses/update.json", post=True, single=True,
-        status=message, source="gwibbernet", in_reply_to_status_id=target["id"])
-
+        status=message, in_reply_to_status_id=target["id"])
