@@ -10,11 +10,9 @@ from const import *
 COUNT = 200
 
 def parsetime(t):
-  locale.setlocale(locale.LC_TIME, '')
-  loc = locale.setlocale(locale.LC_TIME)
   locale.setlocale(locale.LC_TIME, 'C')
   result = mx.DateTime.Parser.DateTimeFromString(t)
-  locale.setlocale(locale.LC_TIME, loc)
+  locale.setlocale(locale.LC_TIME, '')
   return result.ticks()
 
 URL_SCHEMES = ('http', 'https', 'ftp', 'mailto', 'news', 'gopher',
