@@ -26,7 +26,7 @@ except:
     exit
 
 from gwibber import resources
-from gwibber import util
+from gwibber.microblog import util
 import gtk, gconf
 
 import gettext
@@ -83,6 +83,9 @@ class GwibberPreferences(object):
         self.urlshorter_selector = gtk.combo_box_new_text()
         for urlshorter in urlshorters.keys(): self.urlshorter_selector.append_text(urlshorter)
         self.ui.get_object("urlshorter_container").pack_start(self.urlshorter_selector, True, True)
+        if not self.settings["urlshorter"] in urlshorters.keys():
+          self.settings["urlshorter"] = DEFAULT_SETTINGS["urlshorter"]
+          self.settings.save()
         self.urlshorter_selector.set_active_iter(dict([(x[0].strip(), x.iter) for x in self.urlshorter_selector.get_model()]).get(self.settings["urlshorter"], self.urlshorter_selector.get_model().get_iter_root()))
         self.urlshorter_selector.show_all()
 
